package moze_intel.projecte.gameObjs.items;

import java.util.List;

import net.minecraft.block.Block;
import net.minecraft.block.BlockCauldron;
import net.minecraft.client.renderer.texture.IIconRegister;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.init.Blocks;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumChatFormatting;
import net.minecraft.util.MovingObjectPosition;
import net.minecraft.util.StatCollector;
import net.minecraft.world.World;
import net.minecraftforge.fluids.FluidContainerRegistry;
import net.minecraftforge.fluids.FluidRegistry;
import net.minecraftforge.fluids.FluidStack;
import net.minecraftforge.fluids.IFluidContainerItem;
import net.minecraftforge.fluids.IFluidHandler;

import com.google.common.collect.Lists;

import baubles.api.BaubleType;
import baubles.api.IBauble;
import cpw.mods.fml.common.Optional;
import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;
import moze_intel.projecte.api.item.IPedestalItem;
import moze_intel.projecte.api.item.IProjectileShooter;
import moze_intel.projecte.config.ProjectEConfig;
import moze_intel.projecte.gameObjs.entity.EntityWaterProjectile;
import moze_intel.projecte.gameObjs.tiles.DMPedestalTile;
import moze_intel.projecte.utils.ClientKeyHelper;
import moze_intel.projecte.utils.Constants;
import moze_intel.projecte.utils.FluidHelper;
import moze_intel.projecte.utils.MathUtils;
import moze_intel.projecte.utils.PEKeybind;
import moze_intel.projecte.utils.PlayerHelper;

@Optional.Interface(iface = "baubles.api.IBauble", modid = "Baubles")
public class EvertideAmulet extends ItemPE implements IProjectileShooter, IBauble, IPedestalItem, IFluidContainerItem {

    public EvertideAmulet() {
        this.setUnlocalizedName("evertide_amulet");
        this.setMaxStackSize(1);
        this.setNoRepair();
        this.setContainerItem(this);
    }

    @Override
    public boolean onItemUse(ItemStack stack, EntityPlayer player, World world, int x, int y, int z, int sideHit,
        float f1, float f2, float f3) {
        if (!world.isRemote && PlayerHelper.hasEditPermission(((EntityPlayerMP) player), x, y, z)) {
            TileEntity tile = world.getTileEntity(x, y, z);

            if (tile instanceof IFluidHandler) {
                IFluidHandler tank = (IFluidHandler) tile;

                if (FluidHelper.canFillTank(tank, FluidRegistry.WATER, sideHit)) {
                    FluidHelper.fillTank(tank, FluidRegistry.WATER, sideHit, 1000);
                    return true;
                }
            }

            Block block = world.getBlock(x, y, z);
            int meta = world.getBlockMetadata(x, y, z);
            if (block == Blocks.cauldron && meta < 3) {
                ((BlockCauldron) block).func_150024_a(world, x, y, z, meta + 1);
                // Cauldron-specific setblock that has extra checks on metadata, called by vanilla water buckets
            }
        }

        return false;
    }

    @Override
    public ItemStack onItemRightClick(ItemStack stack, World world, EntityPlayer player) {
        if (!world.isRemote) {
            MovingObjectPosition mop = this.getMovingObjectPositionFromPlayer(world, player, false);
            if (mop != null && mop.typeOfHit == MovingObjectPosition.MovingObjectType.BLOCK) {
                int i = mop.blockX;
                int j = mop.blockY;
                int k = mop.blockZ;
                if (!(world.getTileEntity(i, j, k) instanceof IFluidHandler)) {
                    switch (mop.sideHit) // Ripped from vanilla ItemBucket and simplified
                    {
                        case 0:
                            --j;
                            break;
                        case 1:
                            ++j;
                            break;
                        case 2:
                            --k;
                            break;
                        case 3:
                            ++k;
                            break;
                        case 4:
                            --i;
                            break;
                        case 5:
                            ++i;
                            break;
                        default:
                            break;
                    }

                    if (world.isAirBlock(i, j, k)) {
                        world.playSoundAtEntity(player, "projecte:item.pewatermagic", 1.0F, 1.0F);
                        placeWater(world, player, i, j, k);
                        PlayerHelper.swingItem(player);
                    }
                }
            }
        }

        return stack;
    }

    private void placeWater(World world, EntityPlayer player, int i, int j, int k) {
        if (world.provider.isHellWorld) {
            world.playSoundEffect(
                (double) ((float) i + 0.5F),
                (double) ((float) j + 0.5F),
                (double) ((float) k + 0.5F),
                "random.fizz",
                0.5F,
                2.6F + (world.rand.nextFloat() - world.rand.nextFloat()) * 0.8F);

            for (int l = 0; l < 8; ++l) {
                world.spawnParticle(
                    "largesmoke",
                    (double) i + Math.random(),
                    (double) j + Math.random(),
                    (double) k + Math.random(),
                    0.0D,
                    0.0D,
                    0.0D);
            }
        } else {
            PlayerHelper.checkedPlaceBlock(((EntityPlayerMP) player), i, j, k, Blocks.flowing_water, 0);
        }

    }

    @Override
    public void onUpdate(ItemStack stack, World world, Entity entity, int invSlot, boolean par5) {
        if (invSlot > 8 || !(entity instanceof EntityPlayer)) {
            return;
        }

        EntityPlayer player = (EntityPlayer) entity;

        int x = (int) Math.floor(player.posX);
        int y = (int) (player.posY - player.getYOffset());
        int z = (int) Math.floor(player.posZ);

        if ((world.getBlock(x, y - 1, z) == Blocks.water || world.getBlock(x, y - 1, z) == Blocks.flowing_water)
            && world.getBlock(x, y, z) == Blocks.air) {
            if (!player.isSneaking()) {
                player.motionY = 0.0D;
                player.fallDistance = 0.0F;
                player.onGround = true;
            }

            if (!world.isRemote && player.capabilities.getWalkSpeed() < 0.25F) {
                PlayerHelper.setPlayerWalkSpeed(player, 0.25F);
            }
        } else if (!world.isRemote) {
            if (player.isInWater()) {
                player.setAir(300);
            }

            if (player.capabilities.getWalkSpeed() != Constants.PLAYER_WALK_SPEED) {
                PlayerHelper.setPlayerWalkSpeed(player, Constants.PLAYER_WALK_SPEED);
            }
        }
    }

    @Override
    public boolean shootProjectile(EntityPlayer player, ItemStack stack) {
        World world = player.worldObj;

        if (!world.provider.isHellWorld) {
            world.playSoundAtEntity(player, "projecte:item.pewatermagic", 1.0F, 1.0F);
            world.spawnEntityInWorld(new EntityWaterProjectile(world, player));
            return true;
        }

        return false;
    }

    /** Start IFluidContainerItem **/
    @Override
    public FluidStack getFluid(ItemStack container) {
        return new FluidStack(FluidRegistry.WATER, FluidContainerRegistry.BUCKET_VOLUME);
    }

    @Override
    public int getCapacity(ItemStack container) {
        return FluidContainerRegistry.BUCKET_VOLUME;
    }

    @Override
    public int fill(ItemStack container, FluidStack resource, boolean doFill) {
        return 0;
    }

    @Override
    public FluidStack drain(ItemStack container, int maxDrain, boolean doDrain) {
        return new FluidStack(FluidRegistry.WATER, FluidContainerRegistry.BUCKET_VOLUME);
    }

    /** End IFluidContainerItem **/

    @Override
    @SideOnly(Side.CLIENT)
    public void registerIcons(IIconRegister register) {
        this.itemIcon = register.registerIcon(this.getTexture("rings", "evertide_amulet"));// "ee2:rings/evertide_amulet");
    }

    @Override
    @SideOnly(Side.CLIENT)
    public void addInformation(ItemStack stack, EntityPlayer player, List list, boolean par4) {
        list.add(
            String.format(
                StatCollector.translateToLocal("pe.evertide.tooltip1"),
                ClientKeyHelper.getKeyName(PEKeybind.FIRE_PROJECTILE)));

        list.add(StatCollector.translateToLocal("pe.evertide.tooltip2"));
        list.add(StatCollector.translateToLocal("pe.evertide.tooltip3"));
        list.add(StatCollector.translateToLocal("pe.evertide.tooltip4"));
    }

    @Override
    @Optional.Method(modid = "Baubles")
    public baubles.api.BaubleType getBaubleType(ItemStack itemstack) {
        return BaubleType.AMULET;
    }

    @Override
    @Optional.Method(modid = "Baubles")
    public void onWornTick(ItemStack stack, EntityLivingBase player) {
        this.onUpdate(stack, player.worldObj, player, 0, false);
    }

    @Override
    @Optional.Method(modid = "Baubles")
    public void onEquipped(ItemStack itemstack, EntityLivingBase player) {}

    @Override
    @Optional.Method(modid = "Baubles")
    public void onUnequipped(ItemStack itemstack, EntityLivingBase player) {}

    @Override
    @Optional.Method(modid = "Baubles")
    public boolean canEquip(ItemStack itemstack, EntityLivingBase player) {
        return true;
    }

    @Override
    @Optional.Method(modid = "Baubles")
    public boolean canUnequip(ItemStack itemstack, EntityLivingBase player) {
        return true;
    }

    @Override
    public void updateInPedestal(World world, int x, int y, int z) {
        if (!world.isRemote && ProjectEConfig.evertidePedCooldown != -1) {
            DMPedestalTile tile = ((DMPedestalTile) world.getTileEntity(x, y, z));

            if (tile.getActivityCooldown() == 0) {
                int i = (300 + world.rand.nextInt(600)) * 20;
                world.getWorldInfo()
                    .setRainTime(i);
                world.getWorldInfo()
                    .setThunderTime(i);
                world.getWorldInfo()
                    .setRaining(true);

                tile.setActivityCooldown(ProjectEConfig.evertidePedCooldown);
            } else {
                tile.decrementActivityCooldown();
            }
        }
    }

    @Override
    public List<String> getPedestalDescription() {
        List<String> list = Lists.newArrayList();
        if (ProjectEConfig.evertidePedCooldown != -1) {
            list.add(EnumChatFormatting.BLUE + StatCollector.translateToLocal("pe.evertide.pedestal1"));
            list.add(
                EnumChatFormatting.BLUE + String.format(
                    StatCollector.translateToLocal("pe.evertide.pedestal2"),
                    MathUtils.tickToSecFormatted(ProjectEConfig.evertidePedCooldown)));
        }
        return list;
    }
}
