package moze_intel.projecte.gameObjs.items.rings;

import java.util.List;

import net.minecraft.client.renderer.texture.IIconRegister;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLiving;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Items;
import net.minecraft.item.ItemStack;
import net.minecraft.util.EnumChatFormatting;
import net.minecraft.util.IIcon;
import net.minecraft.util.StatCollector;
import net.minecraft.world.World;
import net.minecraft.world.WorldServer;
import net.minecraftforge.common.util.FakePlayerFactory;

import com.google.common.collect.Lists;

import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;
import moze_intel.projecte.PECore;
import moze_intel.projecte.api.item.IModeChanger;
import moze_intel.projecte.api.item.IPedestalItem;
import moze_intel.projecte.config.ProjectEConfig;
import moze_intel.projecte.gameObjs.entity.EntityHomingArrow;
import moze_intel.projecte.gameObjs.tiles.DMPedestalTile;
import moze_intel.projecte.utils.EMCHelper;
import moze_intel.projecte.utils.MathUtils;

public class ArchangelSmite extends RingToggle implements IPedestalItem, IModeChanger {

    public ArchangelSmite() {
        super("archangel_smite");
        this.setMaxStackSize(1);
        this.setNoRepair();
    }

    @Override
    public void onUpdate(ItemStack stack, World world, Entity entity, int par4, boolean par5) {
        if (!world.isRemote && getMode(stack) == 1 && entity instanceof EntityLivingBase) {
            fireArrow(stack, world, ((EntityLivingBase) entity));
        }
    }

    @Override
    public ItemStack onItemRightClick(ItemStack stack, World world, EntityPlayer player) {
        if (!world.isRemote) {
            fireArrow(stack, world, player);
        }
        return stack;
    }

    private void fireArrow(ItemStack ring, World world, EntityLivingBase shooter) {
        EntityHomingArrow arrow = new EntityHomingArrow(world, shooter, 2.0F);

        if (!(shooter instanceof EntityPlayer)
            || consumeFuel(((EntityPlayer) shooter), ring, EMCHelper.getEmcValue(Items.arrow), true)) {
            world.playSoundAtEntity(shooter, "random.bow", 1.0F, 1.0F / (itemRand.nextFloat() * 0.4F + 1.2F) + 0.5F);
            world.spawnEntityInWorld(arrow);
        }
    }

    @Override
    @SideOnly(Side.CLIENT)
    public void registerIcons(IIconRegister register) {
        this.itemIcon = register.registerIcon(this.getTexture("rings", "archangel_smite"));
    }

    @Override
    @SideOnly(Side.CLIENT)
    public IIcon getIconFromDamage(int dmg) {
        return itemIcon;
    }

    @Override
    public void updateInPedestal(World world, int x, int y, int z) {
        if (!world.isRemote && ProjectEConfig.archangelPedCooldown != -1) {
            DMPedestalTile tile = ((DMPedestalTile) world.getTileEntity(x, y, z));
            if (tile.getActivityCooldown() == 0) {
                if (!world.getEntitiesWithinAABB(EntityLiving.class, tile.getEffectBounds())
                    .isEmpty()) {
                    for (int i = 0; i < 3; i++) {
                        EntityHomingArrow arrow = new EntityHomingArrow(
                            world,
                            FakePlayerFactory.get(((WorldServer) world), PECore.FAKEPLAYER_GAMEPROFILE),
                            2.0F);
                        arrow.posX = tile.centeredX;
                        arrow.posY = tile.centeredY + 2;
                        arrow.posZ = tile.centeredZ;
                        arrow.motionX = 0;
                        arrow.motionZ = 0;
                        arrow.motionY = 1;
                        world.playSoundAtEntity(
                            arrow,
                            "random.bow",
                            1.0F,
                            1.0F / (itemRand.nextFloat() * 0.4F + 1.2F) + 0.5F);
                        world.spawnEntityInWorld(arrow);
                    }
                }
                tile.setActivityCooldown(ProjectEConfig.archangelPedCooldown);
            } else {
                tile.decrementActivityCooldown();
            }
        }
    }

    @Override
    public List<String> getPedestalDescription() {
        List<String> list = Lists.newArrayList();
        if (ProjectEConfig.archangelPedCooldown != -1) {
            list.add(EnumChatFormatting.BLUE + StatCollector.translateToLocal("pe.archangel.pedestal1"));
            list.add(
                EnumChatFormatting.BLUE + String.format(
                    StatCollector.translateToLocal("pe.archangel.pedestal2"),
                    MathUtils.tickToSecFormatted(ProjectEConfig.archangelPedCooldown)));
        }
        return list;
    }
}
