package moze_intel.projecte.impl;

import javax.annotation.Nonnull;

import net.minecraft.block.Block;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;

import com.google.common.base.Preconditions;

import cpw.mods.fml.common.Loader;
import cpw.mods.fml.common.LoaderState;
import moze_intel.projecte.api.proxy.IEMCProxy;
import moze_intel.projecte.emc.mappers.APICustomEMCMapper;
import moze_intel.projecte.utils.EMCHelper;
import moze_intel.projecte.utils.PELogger;

public class EMCProxyImpl implements IEMCProxy {

    public static final IEMCProxy instance = new EMCProxyImpl();

    private EMCProxyImpl() {}

    @Override
    public void registerCustomEMC(ItemStack stack, int value) {
        Preconditions.checkNotNull(stack);
        boolean flag = Loader.instance()
            .isInState(LoaderState.PREINITIALIZATION)
            || Loader.instance()
                .isInState(LoaderState.INITIALIZATION)
            || Loader.instance()
                .isInState(LoaderState.POSTINITIALIZATION);
        Preconditions.checkState(
            flag,
            String.format(
                "Mod %s tried to register EMC at an invalid time!",
                Loader.instance()
                    .activeModContainer()
                    .getModId()));
        APICustomEMCMapper.instance.registerCustomEMC(stack, value);
        PELogger.logInfo(
            "Mod %s registered emc value %d for itemstack %s",
            Loader.instance()
                .activeModContainer()
                .getModId(),
            value,
            stack.toString());
    }

    @Override
    public void registerCustomEMC(Object o, int value) {
        Preconditions.checkNotNull(o);
        boolean flag = Loader.instance()
            .isInState(LoaderState.PREINITIALIZATION)
            || Loader.instance()
                .isInState(LoaderState.INITIALIZATION)
            || Loader.instance()
                .isInState(LoaderState.POSTINITIALIZATION);
        Preconditions.checkState(
            flag,
            String.format(
                "Mod %s tried to register EMC at an invalid time!",
                Loader.instance()
                    .activeModContainer()
                    .getModId()));
        APICustomEMCMapper.instance.registerCustomEMC(o, value);
        PELogger.logInfo(
            "Mod %s registered emc value %d for Object %s",
            Loader.instance()
                .activeModContainer()
                .getModId(),
            value,
            o);
    }

    @Override
    public boolean hasValue(Block block) {
        Preconditions.checkNotNull(block);
        return EMCHelper.doesBlockHaveEmc(block);
    }

    @Override
    public boolean hasValue(Item item) {
        Preconditions.checkNotNull(item);
        return EMCHelper.doesItemHaveEmc(item);
    }

    @Override
    public boolean hasValue(ItemStack stack) {
        Preconditions.checkNotNull(stack);
        return EMCHelper.doesItemHaveEmc(stack);
    }

    @Override
    public long getValue(@Nonnull Block block) {
        Preconditions.checkNotNull(block);
        return EMCHelper.getEmcValue(block);
    }

    @Override
    public long getValue(@Nonnull Item item) {
        Preconditions.checkNotNull(item);
        return EMCHelper.getEmcValue(item);
    }

    @Override
    public long getValue(@Nonnull ItemStack stack) {
        Preconditions.checkNotNull(stack);
        return EMCHelper.getEmcValue(stack);
    }
}
