package moze_intel.projecte.utils;

import java.lang.reflect.Constructor;
import java.util.*;

import net.minecraft.block.Block;
import net.minecraft.block.BlockFlower;
import net.minecraft.block.IGrowable;
import net.minecraft.enchantment.Enchantment;
import net.minecraft.enchantment.EnchantmentHelper;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLiving;
import net.minecraft.entity.IProjectile;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.entity.monster.EntityBlaze;
import net.minecraft.entity.monster.EntityCreeper;
import net.minecraft.entity.monster.EntityEnderman;
import net.minecraft.entity.monster.EntityGhast;
import net.minecraft.entity.monster.EntityPigZombie;
import net.minecraft.entity.monster.EntitySilverfish;
import net.minecraft.entity.monster.EntitySkeleton;
import net.minecraft.entity.monster.EntitySlime;
import net.minecraft.entity.monster.EntitySpider;
import net.minecraft.entity.monster.EntityWitch;
import net.minecraft.entity.monster.EntityZombie;
import net.minecraft.entity.monster.IMob;
import net.minecraft.entity.passive.EntityBat;
import net.minecraft.entity.passive.EntityChicken;
import net.minecraft.entity.passive.EntityCow;
import net.minecraft.entity.passive.EntityHorse;
import net.minecraft.entity.passive.EntityMooshroom;
import net.minecraft.entity.passive.EntityOcelot;
import net.minecraft.entity.passive.EntityPig;
import net.minecraft.entity.passive.EntitySheep;
import net.minecraft.entity.passive.EntitySquid;
import net.minecraft.entity.passive.EntityVillager;
import net.minecraft.entity.passive.EntityWolf;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.entity.projectile.EntityArrow;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.util.Vec3;
import net.minecraft.world.World;
import net.minecraftforge.common.IPlantable;
import net.minecraftforge.common.IShearable;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.common.util.ForgeDirection;
import net.minecraftforge.event.world.ExplosionEvent;

import com.google.common.collect.ImmutableList;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;

import moze_intel.projecte.config.ProjectEConfig;
import moze_intel.projecte.gameObjs.entity.EntityLootBall;

/**
 * Helper class for anything that touches a World. Notice: Please try to keep methods tidy and alphabetically ordered.
 * Thanks!
 */
public final class WorldHelper {

    public static final ImmutableList<Class<? extends EntityLiving>> peacefuls = ImmutableList
        .<Class<? extends EntityLiving>>of(
            EntitySheep.class,
            EntityPig.class,
            EntityCow.class,
            EntityMooshroom.class,
            EntityChicken.class,
            EntityBat.class,
            EntityVillager.class,
            EntitySquid.class,
            EntityOcelot.class,
            EntityWolf.class,
            EntityHorse.class);
    public static final ImmutableList<Class<? extends EntityLiving>> mobs = ImmutableList
        .<Class<? extends EntityLiving>>of(
            EntityZombie.class,
            EntitySkeleton.class,
            EntityCreeper.class,
            EntitySpider.class,
            EntityEnderman.class,
            EntitySilverfish.class,
            EntityPigZombie.class,
            EntityGhast.class,
            EntityBlaze.class,
            EntitySlime.class,
            EntityWitch.class);

    public static Set<Class<? extends Entity>> interdictionBlacklist = Sets.newHashSet();

    public static Set<Class<? extends Entity>> swrgBlacklist = Sets.newHashSet();

    public static boolean blacklistInterdiction(Class<? extends Entity> clazz) {
        if (!interdictionBlacklist.contains(clazz)) {
            interdictionBlacklist.add(clazz);
            return true;
        }
        return false;
    }

    public static boolean blacklistSwrg(Class<? extends Entity> clazz) {
        if (!interdictionBlacklist.contains(clazz)) {
            interdictionBlacklist.add(clazz);
            return true;
        }
        return false;
    }

    public static void createLootDrop(List<ItemStack> drops, World world, double x, double y, double z) {
        if (drops.isEmpty()) {
            return;
        }

        ItemHelper.compactItemList(drops);

        if (ProjectEConfig.useLootBalls) {
            world.spawnEntityInWorld(new EntityLootBall(world, drops, x, y, z));
        } else {
            for (ItemStack drop : drops) {
                spawnEntityItem(world, drop, x, y, z);
            }
        }
    }

    /**
     * Equivalent of World.newExplosion
     */
    public static void createNovaExplosion(World world, Entity exploder, double x, double y, double z, float power) {
        NovaExplosion explosion = new NovaExplosion(world, exploder, x, y, z, power);
        if (!MinecraftForge.EVENT_BUS.post(new ExplosionEvent.Start(world, explosion))) {
            explosion.doExplosionA();
            explosion.doExplosionB(true);
        }
    }

    public static void extinguishNearby(World world, EntityPlayer player) {
        for (int x = (int) (player.posX - 1); x <= player.posX + 1; x++) for (int y = (int) (player.posY - 1); y
            <= player.posY + 1; y++)
            for (int z = (int) (player.posZ - 1); z <= player.posZ + 1; z++) if (world.getBlock(x, y, z) == Blocks.fire
                && PlayerHelper.hasBreakPermission(((EntityPlayerMP) player), x, y, z)) {
                    world.setBlockToAir(x, y, z);
                }
    }

    public static void freezeInBoundingBox(World world, AxisAlignedBB box, EntityPlayer player, boolean random) {
        for (int x = (int) box.minX; x <= box.maxX; x++) {
            for (int y = (int) box.minY; y <= box.maxY; y++) {
                for (int z = (int) box.minZ; z <= box.maxZ; z++) {
                    Block b = world.getBlock(x, y, z);

                    if ((b == Blocks.water || b == Blocks.flowing_water) && (!random || world.rand.nextInt(128) == 0)) {
                        if (player != null) {
                            PlayerHelper.checkedReplaceBlock(((EntityPlayerMP) player), x, y, z, Blocks.ice, 0);
                        } else {
                            world.setBlock(x, y, z, Blocks.ice);
                        }
                    } else if (b.isSideSolid(world, x, y, z, ForgeDirection.UP)) {
                        Block b2 = world.getBlock(x, y + 1, z);

                        if (b2 == Blocks.air && (!random || world.rand.nextInt(128) == 0)) {
                            if (player != null) {
                                PlayerHelper
                                    .checkedReplaceBlock(((EntityPlayerMP) player), x, y + 1, z, Blocks.snow_layer, 0);
                            } else {
                                world.setBlock(x, y + 1, z, Blocks.snow_layer);
                            }
                        }
                    }
                }
            }
        }
    }

    public static List<TileEntity> getAdjacentTileEntities(World world, TileEntity tile) {
        return ImmutableList.copyOf(getAdjacentTileEntitiesMapped(world, tile).values());
    }

    public static Map<ForgeDirection, TileEntity> getAdjacentTileEntitiesMapped(final World world,
        final TileEntity tile) {
        Map<ForgeDirection, TileEntity> ret = new EnumMap<>(ForgeDirection.class);

        for (ForgeDirection dir : ForgeDirection.VALID_DIRECTIONS) {
            TileEntity candidate = world
                .getTileEntity(tile.xCoord + dir.offsetX, tile.yCoord + dir.offsetY, tile.zCoord + dir.offsetZ);
            if (candidate != null) {
                ret.put(dir, candidate);
            }
        }

        return ret;
    }

    public static ArrayList<ItemStack> getBlockDrops(World world, EntityPlayer player, Block block, ItemStack stack,
        int x, int y, int z) {
        int meta = world.getBlockMetadata(x, y, z);

        if (EnchantmentHelper.getEnchantmentLevel(Enchantment.silkTouch.effectId, stack) > 0
            && block.canSilkHarvest(world, player, x, y, z, meta)) {
            return Lists.newArrayList(new ItemStack(block, 1, meta));
        }

        return block
            .getDrops(world, x, y, z, meta, EnchantmentHelper.getEnchantmentLevel(Enchantment.fortune.effectId, stack));
    }

    /**
     * Gets an AABB for AOE digging operations. The offset increases both the breadth and depth of the box.
     */
    public static AxisAlignedBB getBroadDeepBox(Coordinates coords, ForgeDirection direction, int offset) {
        if (direction.offsetX > 0) {
            return AxisAlignedBB.getBoundingBox(
                coords.x - offset,
                coords.y - offset,
                coords.z - offset,
                coords.x,
                coords.y + offset,
                coords.z + offset);
        } else if (direction.offsetX < 0) {
            return AxisAlignedBB.getBoundingBox(
                coords.x,
                coords.y - offset,
                coords.z - offset,
                coords.x + offset,
                coords.y + offset,
                coords.z + offset);
        } else if (direction.offsetY > 0) {
            return AxisAlignedBB.getBoundingBox(
                coords.x - offset,
                coords.y - offset,
                coords.z - offset,
                coords.x + offset,
                coords.y,
                coords.z + offset);
        } else if (direction.offsetY < 0) {
            return AxisAlignedBB.getBoundingBox(
                coords.x - offset,
                coords.y,
                coords.z - offset,
                coords.x + offset,
                coords.y + offset,
                coords.z + offset);
        } else if (direction.offsetZ > 0) {
            return AxisAlignedBB.getBoundingBox(
                coords.x - offset,
                coords.y - offset,
                coords.z - offset,
                coords.x + offset,
                coords.y + offset,
                coords.z);
        } else if (direction.offsetZ < 0) {
            return AxisAlignedBB.getBoundingBox(
                coords.x - offset,
                coords.y - offset,
                coords.z,
                coords.x + offset,
                coords.y + offset,
                coords.z + offset);
        }
        return AxisAlignedBB.getBoundingBox(0, 0, 0, 0, 0, 0);
    }

    /**
     * Returns in AABB that is always 3x3 orthogonal to the side hit, but varies in depth in the direction of the side
     * hit
     */
    public static AxisAlignedBB getDeepBox(Coordinates coords, ForgeDirection direction, int depth) {
        if (direction.offsetX != 0) {
            if (direction.offsetX > 0) {
                return AxisAlignedBB
                    .getBoundingBox(coords.x - depth, coords.y - 1, coords.z - 1, coords.x, coords.y + 1, coords.z + 1);
            } else return AxisAlignedBB
                .getBoundingBox(coords.x, coords.y - 1, coords.z - 1, coords.x + depth, coords.y + 1, coords.z + 1);
        } else if (direction.offsetY != 0) {
            if (direction.offsetY > 0) {
                return AxisAlignedBB
                    .getBoundingBox(coords.x - 1, coords.y - depth, coords.z - 1, coords.x + 1, coords.y, coords.z + 1);
            } else return AxisAlignedBB
                .getBoundingBox(coords.x - 1, coords.y, coords.z - 1, coords.x + 1, coords.y + depth, coords.z + 1);
        } else {
            if (direction.offsetZ > 0) {
                return AxisAlignedBB
                    .getBoundingBox(coords.x - 1, coords.y - 1, coords.z - depth, coords.x + 1, coords.y + 1, coords.z);
            } else return AxisAlignedBB
                .getBoundingBox(coords.x - 1, coords.y - 1, coords.z, coords.x + 1, coords.y + 1, coords.z + depth);
        }
    }

    /**
     * Gets an AABB for AOE digging operations. The charge increases only the breadth of the box. Y level remains
     * constant. As such, a direction hit is unneeded.
     */
    public static AxisAlignedBB getFlatYBox(Coordinates coords, int offset) {
        return AxisAlignedBB.getBoundingBox(
            coords.x - offset,
            coords.y,
            coords.z - offset,
            coords.x + offset,
            coords.y,
            coords.z + offset);
    }

    public static <T extends Entity> T getNewEntityInstance(Class<T> c, World world) {
        try {
            Constructor<T> constr = c.getConstructor(World.class);
            T ent = constr.newInstance(world);

            if (ent instanceof EntitySkeleton) {
                if (world.rand.nextInt(2) == 0) {
                    ((EntitySkeleton) ent).setSkeletonType(1);
                    ent.setCurrentItemOrArmor(0, new ItemStack(Items.stone_sword));
                } else {
                    ent.setCurrentItemOrArmor(0, new ItemStack(Items.bow));
                }
            } else if (ent instanceof EntityPigZombie) {
                ent.setCurrentItemOrArmor(0, new ItemStack(Items.golden_sword));
            }

            return ent;
        } catch (Exception e) {
            PELogger.logFatal("Could not create new entity instance for: " + c.getCanonicalName());
            e.printStackTrace();
        }

        return null;
    }

    public static EntityLiving getRandomEntity(World world, EntityLiving toRandomize) {
        Class<? extends EntityLiving> entClass = toRandomize.getClass();

        if (peacefuls.contains(entClass)) {
            return getNewEntityInstance(CollectionHelper.getRandomListEntry(peacefuls, entClass), world);
        } else if (mobs.contains(entClass)) {
            return getNewEntityInstance(CollectionHelper.getRandomListEntry(mobs, entClass), world);
        } else if (world.rand.nextInt(2) == 0) {
            return new EntitySlime(world);
        } else {
            return new EntitySheep(world);
        }
    }

    public static List<TileEntity> getTileEntitiesWithinAABB(World world, AxisAlignedBB bBox) {
        List<TileEntity> list = Lists.newArrayList();

        for (int i = (int) bBox.minX; i <= bBox.maxX; i++)
            for (int j = (int) bBox.minY; j <= bBox.maxY; j++) for (int k = (int) bBox.minZ; k <= bBox.maxZ; k++) {
                TileEntity tile = world.getTileEntity(i, j, k);
                if (tile != null) {
                    list.add(tile);
                }
            }

        return list;
    }

    /**
     * Gravitates an entity, vanilla xp orb style, towards a position Code adapted from EntityXPOrb and OpenBlocks
     * Vacuum Hopper, mostly the former
     */
    public static void gravitateEntityTowards(Entity ent, double x, double y, double z) {
        double dX = x - ent.posX;
        double dY = y - ent.posY;
        double dZ = z - ent.posZ;
        double dist = Math.sqrt(dX * dX + dY * dY + dZ * dZ);

        double vel = 1.0 - dist / 15.0;
        if (vel > 0.0D) {
            vel *= vel;
            ent.motionX += dX / dist * vel * 0.05;
            ent.motionY += dY / dist * vel * 0.1;
            ent.motionZ += dZ / dist * vel * 0.05;
            ent.moveEntity(ent.motionX, ent.motionY, ent.motionZ);
        }
    }

    public static void growNearbyRandomly(boolean harvest, World world, double xCoord, double yCoord, double zCoord,
        EntityPlayer player) {
        int chance = harvest ? 16 : 32;

        for (int x = (int) (xCoord - 5); x <= xCoord + 5; x++) for (int y = (int) (yCoord - 3); y <= yCoord + 3; y++)
            for (int z = (int) (zCoord - 5); z <= zCoord + 5; z++) {
                Block crop = world.getBlock(x, y, z);

                // Vines, leaves, tallgrass, deadbush, doubleplants
                if (crop instanceof IShearable) {
                    if (harvest) {
                        if (player != null && PlayerHelper.hasBreakPermission(((EntityPlayerMP) player), x, y, z)) {
                            world.func_147480_a(x, y, z, true);
                        } else if (player == null) {
                            world.func_147480_a(x, y, z, true);
                        }
                    }
                }
                // Carrot, cocoa, wheat, grass (creates flowers and tall grass in vicinity),
                // Mushroom, potato, sapling, stems, tallgrass
                else if (crop instanceof IGrowable) {
                    IGrowable growable = (IGrowable) crop;
                    if (harvest && !growable.func_149851_a(world, x, y, z, false)) {
                        if (player != null && PlayerHelper.hasBreakPermission(((EntityPlayerMP) player), x, y, z)) {
                            world.func_147480_a(x, y, z, true);
                        } else if (player == null) {
                            world.func_147480_a(x, y, z, true);
                        }
                    } else if (world.rand.nextInt(chance) == 0) {
                        if (ProjectEConfig.harvBandGrass || !crop.getUnlocalizedName()
                            .toLowerCase(Locale.ROOT)
                            .contains("grass")) {
                            growable.func_149853_b(world, world.rand, x, y, z);
                        }
                    }
                }
                // All modded
                // Cactus, Reeds, Netherwart, Flower
                else if (crop instanceof IPlantable) {
                    if (world.rand.nextInt(chance / 4) == 0) {
                        for (int i = 0; i < (harvest ? 8 : 4); i++) {
                            crop.updateTick(world, x, y, z, world.rand);
                        }
                    }

                    if (harvest) {
                        if (crop instanceof BlockFlower) {
                            if (player != null && PlayerHelper.hasBreakPermission(((EntityPlayerMP) player), x, y, z)) {
                                world.func_147480_a(x, y, z, true);
                            } else if (player == null) {
                                world.func_147480_a(x, y, z, true);
                            }
                        }
                        if (crop == Blocks.reeds || crop == Blocks.cactus) {
                            boolean shouldHarvest = true;

                            for (int i = 1; i < 3; i++) {
                                if (world.getBlock(x, y + i, z) != crop) {
                                    shouldHarvest = false;
                                    break;
                                }
                            }

                            if (shouldHarvest) {
                                for (int i = crop == Blocks.reeds ? 1 : 0; i < 3; i++) {
                                    if (player != null
                                        && PlayerHelper.hasBreakPermission(((EntityPlayerMP) player), x, y + i, z)) {
                                        world.func_147480_a(x, y + i, z, true);
                                    } else if (player == null) {
                                        world.func_147480_a(x, y + i, z, true);
                                    }
                                }
                            }
                        }
                        if (crop == Blocks.nether_wart) {
                            int meta = ((IPlantable) crop).getPlantMetadata(world, x, y, z);
                            if (meta == 3) {
                                if (player != null
                                    && PlayerHelper.hasBreakPermission(((EntityPlayerMP) player), x, y, z)) {
                                    world.func_147480_a(x, y, z, true);
                                } else if (player == null) {
                                    world.func_147480_a(x, y, z, true);
                                }
                            }
                        }
                    }
                }
            }
    }

    /**
     * Recursively mines out a vein of the given Block, starting from the provided coordinates
     */
    public static void harvestVein(World world, EntityPlayer player, ItemStack stack, Coordinates coords, Block target,
        List<ItemStack> currentDrops, int numMined) {
        if (numMined >= Constants.MAX_VEIN_SIZE) {
            return;
        }

        AxisAlignedBB b = AxisAlignedBB
            .getBoundingBox(coords.x - 1, coords.y - 1, coords.z - 1, coords.x + 1, coords.y + 1, coords.z + 1);

        for (int x = (int) b.minX; x <= b.maxX; x++)
            for (int y = (int) b.minY; y <= b.maxY; y++) for (int z = (int) b.minZ; z <= b.maxZ; z++) {
                Block block = world.getBlock(x, y, z);

                if (block == target || (target == Blocks.lit_redstone_ore && block == Blocks.redstone_ore)) {
                    numMined++;
                    if (PlayerHelper.hasBreakPermission(((EntityPlayerMP) player), x, y, z)) {
                        currentDrops.addAll(getBlockDrops(world, player, block, stack, x, y, z));
                        world.setBlockToAir(x, y, z);
                        harvestVein(world, player, stack, new Coordinates(x, y, z), target, currentDrops, numMined);
                    }
                }
            }
    }

    public static void igniteNearby(World world, EntityPlayer player) {
        for (int x = (int) (player.posX - 8); x <= player.posX + 8; x++)
            for (int y = (int) (player.posY - 5); y <= player.posY + 5; y++)
                for (int z = (int) (player.posZ - 8); z <= player.posZ + 8; z++)
                    if (world.rand.nextInt(128) == 0 && world.isAirBlock(x, y, z)) {
                        PlayerHelper.checkedPlaceBlock(((EntityPlayerMP) player), x, y, z, Blocks.fire, 0);
                    }
    }

    public static boolean isArrowInGround(EntityArrow arrow) {
        return ReflectionHelper.getArrowInGround(arrow);
    }

    /**
     * Repels projectiles and mobs in the given AABB away from a given point
     */
    public static void repelEntitiesInAABBFromPoint(World world, AxisAlignedBB effectBounds, double x, double y,
        double z, boolean isSWRG) {
        List<Entity> list = world.getEntitiesWithinAABB(Entity.class, effectBounds);

        for (Entity ent : list) {
            if ((isSWRG && !swrgBlacklist.contains(ent.getClass()))
                || (!isSWRG && !interdictionBlacklist.contains(ent.getClass()))) {
                if ((ent instanceof EntityLiving) || (ent instanceof IProjectile)) {
                    if (!isSWRG && ProjectEConfig.interdictionMode
                        && !(ent instanceof IMob || ent instanceof IProjectile)) {
                        continue;
                    } else {
                        if (ent instanceof EntityArrow && ((EntityArrow) ent).onGround) {
                            continue;
                        }
                        Vec3 p = Vec3.createVectorHelper(x, y, z);
                        Vec3 t = Vec3.createVectorHelper(ent.posX, ent.posY, ent.posZ);
                        double distance = p.distanceTo(t) + 0.1D;

                        Vec3 r = Vec3.createVectorHelper(t.xCoord - p.xCoord, t.yCoord - p.yCoord, t.zCoord - p.zCoord);

                        ent.motionX += r.xCoord / 1.5D / distance;
                        ent.motionY += r.yCoord / 1.5D / distance;
                        ent.motionZ += r.zCoord / 1.5D / distance;
                    }
                }
            }
        }
    }

    public static void spawnEntityItem(World world, ItemStack stack, double x, double y, double z) {
        float f = world.rand.nextFloat() * 0.8F + 0.1F;
        float f1 = world.rand.nextFloat() * 0.8F + 0.1F;
        EntityItem entityitem;

        for (float f2 = world.rand.nextFloat() * 0.8F + 0.1F; stack.stackSize > 0; world
            .spawnEntityInWorld(entityitem)) {
            int j1 = world.rand.nextInt(21) + 10;

            if (j1 > stack.stackSize) j1 = stack.stackSize;

            stack.stackSize -= j1;
            entityitem = new EntityItem(
                world,
                (double) ((float) x + f),
                (double) ((float) y + f1),
                (double) ((float) z + f2),
                new ItemStack(stack.getItem(), j1, stack.getItemDamage()));
            float f3 = 0.05F;
            entityitem.motionX = (double) ((float) world.rand.nextGaussian() * f3);
            entityitem.motionY = (double) ((float) world.rand.nextGaussian() * f3 + 0.2F);
            entityitem.motionZ = (double) ((float) world.rand.nextGaussian() * f3);

            if (stack.hasTagCompound()) {
                entityitem.getEntityItem()
                    .setTagCompound(
                        (NBTTagCompound) stack.getTagCompound()
                            .copy());
            }
        }

    }
}
